@@ -30,7 +30,7 @@
 #include "slave.h"
 #include "verify.h"
 
-static const gchar RCSid[]="$Id: verify-crypt.c,v 1.2 2001/01/10 07:30:21 jirka Exp $";
+static const gchar RCSid[]="$Id: verify-crypt.c,v 1.4 2001/01/31 01:16:44 jirka Exp $";
 
 
 /* Configuration option variables */
@@ -61,7 +61,7 @@ gdm_verify_user (const gchar *display)
     ppasswd = !pwent ? NULL : pwent->pw_passwd;
     
     /* Request the user's password */
-    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password:"));
+    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password: "));
 
     /* If verbose authentication is enabled, output messages from the
      * authentication system */
@@ -69,7 +69,7 @@ gdm_verify_user (const gchar *display)
 
 	if (!pwent) {
 	    gdm_error (_("Couldn't authenticate %s"), login);
-	    gdm_slave_greeter_ctl (GDM_MSGERR, _("User unknown"));
+	    gdm_slave_greeter_ctl (GDM_MSGERR, _("Login incorrect"));
 	    return NULL;
 	}
     
@@ -84,7 +84,7 @@ gdm_verify_user (const gchar *display)
     if (!passwd || !ppasswd || strcmp (crypt (passwd, ppasswd), ppasswd)) {
 
 	if (GdmVerboseAuth)
-	    gdm_slave_greeter_ctl (GDM_MSGERR, _("Incorrect password"));
+	    gdm_slave_greeter_ctl (GDM_MSGERR, _("Login incorrect"));
 	
 	return NULL;
     }
