@@ -35,12 +35,14 @@
 #include <X11/Xlib.h>
 #include <pwd.h>
 #include <sys/utsname.h>
-
+#ifdef HAVE_LIBXINERAMA
+#include <X11/extensions/Xinerama.h>
+#endif
 #include "gdmlogin.h"
 #include "gdm.h"
 #include "filecheck.h"
 
-static const gchar RCSid[]="$Id: gdmlogin.c,v 1.19 2001/01/26 04:00:51 jirka Exp $";
+static const gchar RCSid[]="$Id: gdmlogin.c,v 1.26 2001/02/01 23:23:04 jirka Exp $";
 
 static gboolean GdmBrowser;
 static gboolean GdmDebug;
@@ -50,6 +52,7 @@ static gboolean GdmQuiver;
 static gint  GdmRelaxPerms;
 static gboolean GdmSystemMenu;
 static gint  GdmUserMaxFile;
+static gint  GdmXineramaScreen;
 static gchar *GdmLogo;
 static gchar *GdmWelcome;
 static gchar *GdmBackgroundProg;
@@ -77,6 +80,7 @@ static GtkWidget *sessmenu;
 static GtkWidget *langmenu;
 static GdkWindow *rootwin;
 static GdkWindow *rootwin_overlay = NULL;
+static GdkRectangle screen;
 
 static GnomeIconList *browser;
 static GdkImlibImage *defface;
@@ -102,6 +106,40 @@ static gint maxwidth;
 
 static pid_t backgroundpid = 0;
 
+static GHashTable *back_locales = NULL;
+
+static const char *
+gdm_lookup_locale_name (const char *locale)
+{
+	const char *ret;
+	char *shortname;
+	char *p;
+
+	if (locale == NULL ||
+	    back_locales == NULL)
+		return locale;
+
+	ret = g_hash_table_lookup (back_locales, locale);
+	if (ret != NULL)
+		return ret;
+
+	/* Cut the string at the dot */
+	shortname = g_strdup (locale);
+	p = strchr (shortname, '.');
+	if (p == NULL) {
+		g_free (shortname);
+		return locale;
+	}
+	*p = '\0';
+
+	ret = g_hash_table_lookup (back_locales, shortname);
+
+	if (ret != NULL)
+		return ret;
+	else
+		return locale;
+}
+
 static void
 gdm_greeter_chld (int sig)
 {
@@ -328,15 +366,34 @@ gdm_parse_enriched_string (const gchar *s)
     gethostname (hostbuf, 255);
     hostname = g_strdup (hostbuf);
     
-    if (hostname != NULL) 
+    if (hostname == NULL) 
 	hostname = g_strdup ("Gnome");
 
     uname (&name);
 
+    /* HAAAAAAAAAAAAAAAAAAAAAAAACK!, do not translate the next line!,
+     * Since this is the default string we might as well use the gettext
+     * translation as we will likely have better translations there.
+     * Yes ugly as fuck, but oh well, unfortunately two standard defaults
+     * are in circulation */
+    if (strcmp (s, "Welcome to %h") == 0) {
+	g_free (display);
+	buffer = g_strdup_printf (_("Welcome to %s"), hostname);
+	g_free (hostname);
+	return buffer;
+    } else if (strcmp (s, "Welcome to %n") == 0) {
+	g_free (display);
+	g_free (hostname);
+	buffer = g_strdup_printf (_("Welcome to %s"), name.nodename);
+	return buffer;
+    }
+
     if (strlen (s) > 1023) {
 	syslog (LOG_ERR, _("gdm_parse_enriched_string: String too long!"));
 	g_free (display);
-	return (g_strdup_printf (_("Welcome to %s"), hostname));
+	buffer = g_strdup_printf (_("Welcome to %s"), hostname);
+	g_free (hostname);
+	return buffer;
     }
 
     str = g_string_new (NULL);
@@ -395,6 +452,18 @@ gdm_parse_enriched_string (const gchar *s)
     return buffer;
 }
 
+static void
+gdm_center_window (GtkWindow *cw) 
+{
+	GtkRequisition req;
+        gint x, y;
+
+	gtk_widget_size_request (GTK_WIDGET (cw), &req);
+
+	x = screen.x + (screen.width - req.width)/2;
+	y = screen.y + (screen.height - req.height)/2;	
+ 	gtk_widget_set_uposition (GTK_WIDGET (cw), x, y);	
+}
 
 static gboolean
 gdm_login_query (const gchar *msg)
@@ -408,8 +477,7 @@ gdm_login_query (const gchar *msg)
 				 NULL);
 	    
     gtk_window_set_modal (GTK_WINDOW (req), TRUE);
-    gtk_window_set_position (GTK_WINDOW (req), GTK_WIN_POS_CENTER);
-
+    gdm_center_window (GTK_WINDOW (req));
     return (!gnome_dialog_run (GNOME_DIALOG(req)));
 }
 
@@ -473,10 +541,11 @@ gdm_login_parse_config (void)
     GdmDebug = gnome_config_get_bool (GDM_KEY_DEBUG);
     GdmIconMaxWidth = gnome_config_get_int (GDM_KEY_ICONWIDTH);
     GdmIconMaxHeight = gnome_config_get_int (GDM_KEY_ICONHEIGHT);
+    GdmXineramaScreen = gnome_config_get_int (GDM_KEY_XINERAMASCREEN);
     GdmLockPosition = gnome_config_get_bool (GDM_KEY_LOCK_POSITION);
     GdmSetPosition = gnome_config_get_bool (GDM_KEY_SET_POSITION);
-    GdmPositionX = gnome_config_get_int (GDM_KEY_LOCK_POSITION);
-    GdmPositionY = gnome_config_get_int (GDM_KEY_SET_POSITION);
+    GdmPositionX = gnome_config_get_int (GDM_KEY_POSITIONX);
+    GdmPositionY = gnome_config_get_int (GDM_KEY_POSITIONY);
 
     gnome_config_pop_prefix();
 
@@ -593,13 +662,24 @@ gdm_login_language_lookup (const gchar* savedlang)
     gtk_widget_set_sensitive (GTK_WIDGET (langmenu), FALSE);
 
     /* Previously saved language not found in ~user/.gnome/gdm */
-    if (savedlang && ! strlen (savedlang)) {
-	/* If "Last" is chosen use Default, else use current selection */
+    if (savedlang == NULL ||
+	savedlang[0] == '\0') {
+	/* If "Last" is chosen use Default, which is the current language,
+	 * or the GdmDefaultLocale if that's not set or is "C"
+	 * else use current selection */
 	g_free (language);
-	if (g_strcasecmp (curlang, lastlang) == 0)
-	    language = g_strdup (GdmDefaultLocale);
-	else
-	    language = g_strdup (curlang);
+	if (g_strcasecmp (curlang, lastlang) == 0) {
+		char *lang = g_getenv ("LANG");
+		if (lang == NULL ||
+		    lang[0] == '\0' ||
+		    g_strcasecmp (lang, "C") == 0) {
+			language = g_strdup (GdmDefaultLocale);
+		} else {
+			language = g_strdup (lang);
+		}
+	} else {
+		language = g_strdup (curlang);
+	}
 
 	savelang = TRUE;
 	return;
@@ -616,7 +696,9 @@ gdm_login_language_lookup (const gchar* savedlang)
 
 	    msg = g_strdup_printf (_("You have chosen %s for this session, but your default setting is " \
 				     "%s.\nDo you wish to make %s the default for future sessions?"),
-				   curlang, savedlang, curlang);
+				   gdm_lookup_locale_name (curlang),
+				   gdm_lookup_locale_name (savedlang),
+				   gdm_lookup_locale_name (curlang));
 	    savelang = gdm_login_query (msg);
 	    g_free (msg);
 	}
@@ -624,9 +706,6 @@ gdm_login_language_lookup (const gchar* savedlang)
 	g_free (language);
 	language = g_strdup (savedlang);
     }
-
-    /* Now this is utterly ugly, but I suppose it works */
-    language[0] = tolower (language[0]);
 }
 
 static int dance_handler = 0;
@@ -673,6 +752,7 @@ evil (const char *user)
 	if (dance_handler == 0 &&
 	    strcmp (user, "Start Dancing") == 0) {
 		dance_handler = gtk_timeout_add (50, dance, NULL);
+		GdmLockPosition = TRUE;
 	}
 }
 
@@ -830,8 +910,9 @@ gdm_login_language_handler (GtkWidget *widget)
     if (!widget)
 	return;
 
-    gtk_label_get (GTK_LABEL (GTK_BIN (widget)->child), &curlang);
-    s = g_strdup_printf (_("%s language selected"), curlang);
+    curlang = gtk_object_get_data (GTK_OBJECT (widget), "Language");
+    s = g_strdup_printf (_("%s language selected"),
+			 gdm_lookup_locale_name (curlang));
     gtk_label_set (GTK_LABEL (msg), s);
     g_free (s);
 }
@@ -843,7 +924,8 @@ gdm_login_language_init (GtkWidget *menu)
     GtkWidget *item, *ammenu, *nzmenu, *omenu;
     FILE *langlist;
     char curline[256];
-    char *ctmp, *ctmp1, *ctmp2;
+    gboolean has_other_locale = FALSE;
+    GtkWidget *other_menu;
 
     if (!menu)
 	return;
@@ -858,6 +940,9 @@ gdm_login_language_init (GtkWidget *menu)
 			GTK_SIGNAL_FUNC (gdm_login_language_handler), 
 			NULL);
     gtk_widget_show (GTK_WIDGET (item));
+    gtk_object_set_data (GTK_OBJECT (item),
+			 "Language",
+			 lastlang);
 
     item = gtk_menu_item_new();
     gtk_menu_append (GTK_MENU (menu), item);
@@ -875,7 +960,7 @@ gdm_login_language_init (GtkWidget *menu)
     gtk_menu_append (GTK_MENU (menu), item);
     gtk_widget_show(GTK_WIDGET (item));
 
-    item = gtk_menu_item_new_with_label (_("Other"));
+    other_menu = item = gtk_menu_item_new_with_label (_("Other"));
     omenu = gtk_menu_new();
     gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), omenu);
     gtk_menu_append (GTK_MENU (menu), item);
@@ -887,38 +972,51 @@ gdm_login_language_init (GtkWidget *menu)
 	return;
         
     while (fgets (curline, sizeof (curline), langlist)) {
+	    char *name;
+	    char *lang;
 
-	if (!isalpha (curline[0])) 
-	    continue;
+	    if (!isalpha (curline[0])) 
+		    continue;
+
+	    name = strtok (curline, " \t");
+	    if (name == NULL)
+		    continue;
+
+	    lang = strtok (NULL, " \t");
+	    if (lang == NULL)
+		    continue;
+
+	    name[0] = toupper (name[0]);
+
+	    if (back_locales == NULL)
+		    back_locales = g_hash_table_new (g_str_hash, g_str_equal);
+	    g_hash_table_insert (back_locales,
+				 g_strdup (lang),
+				 g_strdup (name));
 	
-	ctmp1 = strchr (curline, ' ');
-	ctmp2 = strchr (curline, '\t');
-	ctmp = curline + strlen (curline) - 1;
+	    item = gtk_radio_menu_item_new_with_label (languages, name);
+	    languages = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (item));
+	    gtk_object_set_data_full (GTK_OBJECT (item),
+				      "Language",
+				      g_strdup (lang),
+				      (GtkDestroyNotify) g_free);
 
-	if (ctmp1 && (ctmp1 < ctmp))
-	    ctmp = ctmp1;
-
-	if (ctmp2 && (ctmp2 < ctmp))
-	    ctmp = ctmp2;
-
-	*ctmp = '\0';
-	curline[0] = toupper (curline[0]);
-	
-	item = gtk_radio_menu_item_new_with_label (languages, curline);
-	languages = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (item));
-
-	if (curline[0] >= 'A' && curline[0] <= 'M')
-	    gtk_menu_append (GTK_MENU (ammenu), item);
-	else if (curline[0] >= 'N' && curline[0] <= 'Z')
-	    gtk_menu_append (GTK_MENU (nzmenu), item);
-	else
-	    gtk_menu_append (GTK_MENU (omenu), item);
+	    if (curline[0] >= 'A' && curline[0] <= 'M') {
+		    gtk_menu_append (GTK_MENU (ammenu), item);
+	    } else if (curline[0] >= 'N' && curline[0] <= 'Z') {
+		    gtk_menu_append (GTK_MENU (nzmenu), item);
+	    } else {
+		    gtk_menu_append (GTK_MENU (omenu), item);
+		    has_other_locale = TRUE;
+	    }
 
 	gtk_signal_connect (GTK_OBJECT (item), "activate", 
 			    GTK_SIGNAL_FUNC (gdm_login_language_handler), 
 			    NULL);
 	gtk_widget_show (GTK_WIDGET (item));
     }
+    if ( ! has_other_locale) 
+	    gtk_widget_destroy (other_menu);
     
     fclose (langlist);
 }
@@ -1001,6 +1099,8 @@ gdm_login_ctrl_handler (GIOChannel *source, GIOCondition cond, gint fd)
 	buf[len-1] = '\0';
 	gtk_label_set (GTK_LABEL (msg), buf);
 	gtk_widget_show (GTK_WIDGET (msg));
+	g_print ("%c\n", STX);
+
 	break;
 
     case GDM_SESS:
@@ -1044,11 +1144,19 @@ gdm_login_ctrl_handler (GIOChannel *source, GIOCondition cond, gint fd)
 	if (GdmQuiver) {
 	    gdk_window_get_position (login->window, &x, &y);
 	    
-	    for (i=32 ; i > 0 ; i=i/2) {
+	    for (i=32 ; i > 0 ; i=i/4) {
 		gdk_window_move (login->window, i+x, y);
+		gdk_flush ();
+		usleep (200);
 		gdk_window_move (login->window, x, y);
+		gdk_flush ();
+		usleep (200);
 		gdk_window_move (login->window, -i+x, y);
+		gdk_flush ();
+		usleep (200);
 		gdk_window_move (login->window, x, y);
+		gdk_flush ();
+		usleep (200);
 	    }
 	}
 
@@ -1304,7 +1412,6 @@ gdm_login_handle_motion (GtkWidget *widget, GdkEventMotion *event)
     GdmSetPosition = TRUE;
     GdmPositionX = xp - p->x;
     GdmPositionY = yp - p->y;
-    gtk_window_position (GTK_WINDOW (login), GTK_WIN_POS_NONE);
 
     return TRUE;
 }
@@ -1412,6 +1519,7 @@ gdm_login_gui_init (void)
     if(*GdmGtkRC)
 	gtk_rc_parse (GdmGtkRC);
 
+
     rootwin = gdk_window_foreign_new (GDK_ROOT_WINDOW ());
 
     gdm_init_root_window_overlay ();
@@ -1555,8 +1663,8 @@ gdm_login_gui_init (void)
 
 	/* FIXME */
 	gtk_widget_set_usize (GTK_WIDGET (bbox),
-			      (gint) gdk_screen_width () * 0.5,
-			      (gint) gdk_screen_height () * 0.25);
+			      (gint) screen.width * 0.5,
+			      (gint) screen.height * 0.25);
     }
 
     if (GdmLogo && !access (GdmLogo, R_OK)) {
@@ -1690,7 +1798,7 @@ gdm_login_gui_init (void)
     if (GdmSetPosition) {
 	    set_screen_pos (login, GdmPositionX, GdmPositionY);
     } else {
-	    gtk_window_position (GTK_WINDOW (login), GTK_WIN_POS_CENTER);
+	    gdm_center_window (GTK_WINDOW (login));
     }
 
     gtk_widget_show_all (GTK_WIDGET (login));
@@ -1863,6 +1971,39 @@ gdm_login_users_init (void)
 }
 
 
+static void 
+gdm_screen_init (void) 
+{
+#ifdef HAVE_LIBXINERAMA
+    if (XineramaIsActive (GDK_DISPLAY ())) {
+	int screen_num;
+	XineramaScreenInfo *screens = XineramaQueryScreens (GDK_DISPLAY (),
+							    &screen_num);
+
+
+	if (screen_num <= 0) 
+	    gdm_login_abort ("Xinerama active, but <= 0 screens?");
+
+	if (screen_num < GdmXineramaScreen)
+	    GdmXineramaScreen = 0;
+
+	screen.x = screens[GdmXineramaScreen].x_org;
+	screen.y = screens[GdmXineramaScreen].y_org;
+	screen.width = screens[GdmXineramaScreen].width;
+	screen.height = screens[GdmXineramaScreen].height;
+	XFree (screens);
+    }
+    else 
+#endif
+    {
+	screen.x = 0;
+	screen.y = 0;
+	screen.width = gdk_screen_width ();
+	screen.height = gdk_screen_height ();
+    }
+}
+
+
 int 
 main (int argc, char *argv[])
 {
@@ -1891,17 +2032,20 @@ main (int argc, char *argv[])
     bindtextdomain (PACKAGE, GNOMELOCALEDIR);
     textdomain (PACKAGE);
 
+    /* XXX: with our own centering this should not be neccessary, is it? */
     gnome_preferences_set_dialog_position (GTK_WIN_POS_CENTER);
     
     bindtextdomain (PACKAGE, GNOMELOCALEDIR);
     textdomain (PACKAGE);
 
-    gdm_login_parse_config();
+    gdm_login_parse_config ();
+
+    gdm_screen_init ();
 
     if (GdmBrowser)
 	gdm_login_users_init ();
 
-    gdm_login_gui_init();
+    gdm_login_gui_init ();
 
     if (GdmBrowser)
 	gdm_login_browser_update();
@@ -1962,10 +2106,11 @@ main (int argc, char *argv[])
 		    NULL);
     g_io_channel_unref (ctrlch);
 
-    gtk_main();
+    gtk_main ();
 
     kill_background ();
-    exit(EXIT_SUCCESS);
+
+    return EXIT_SUCCESS;
 }
 
 /* EOF */
